import { Vector2, WebKitGestureEvent, DomEvents } from '../types';
/**
 * Whether the browser supports GestureEvent (ie Safari)
 * @returns true if the browser supports gesture event
 */
export declare function supportsGestureEvents(): boolean;
export declare function supportsTouchEvents(): boolean;
export declare function getGenericEventData(event: DomEvents): {
    touches: any;
    down: boolean;
    buttons: any;
    shiftKey: any;
    altKey: any;
    metaKey: any;
    ctrlKey: any;
};
/**
 * Gets pointer event values.
 * @param event
 * @returns pointer event values
 */
export declare function getPointerEventValues(event: TouchEvent | PointerEvent): Vector2;
/**
 * Gets scroll event values
 * @param event
 * @returns scroll event values
 */
export declare function getScrollEventValues(event: UIEvent): Vector2;
/**
 * Gets wheel event values.
 * @param event
 * @returns wheel event values
 */
export declare function getWheelEventValues(event: WheelEvent): Vector2;
/**
 * Gets webkit gesture event values.
 * @param event
 * @returns webkit gesture event values
 */
export declare function getWebkitGestureEventValues(event: WebKitGestureEvent): Vector2;
/**
 * Gets two touches event data
 * @param event
 * @returns two touches event data
 */
export declare function getTwoTouchesEventData(event: TouchEvent): {
    values: Vector2;
    origin: Vector2;
};
