(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue-demi')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue-demi'], factory) :
  (global = global || self, factory(global.VueUseGesture = {}, global.vueDemi));
}(this, (function (exports, vueDemi) { 'use strict';

  var global = typeof self !== undefined ? self : this;

  // vector add
  function addV(v1, v2) {
    return v1.map(function (v, i) {
      return v + v2[i];
    });
  } // vector substract

  function subV(v1, v2) {
    return v1.map(function (v, i) {
      return v - v2[i];
    });
  }
  /**
   * Calculates distance
   * @param movement the difference between current and initial vectors
   * @returns distance
   */

  function calculateDistance(movement) {
    return Math.hypot.apply(Math, movement);
  }
  function calculateAllGeometry(movement, delta) {
    if (delta === void 0) {
      delta = movement;
    }

    var dl = calculateDistance(delta);
    var alpha = dl === 0 ? 0 : 1 / dl;
    var direction = delta.map(function (v) {
      return alpha * v;
    });
    var distance = calculateDistance(movement);
    return {
      distance: distance,
      direction: direction
    };
  }
  /**
   * Calculates all kinematics
   * @template T the expected vector type
   * @param movement the difference between current and initial vectors
   * @param delta the difference between current and previous vectors
   * @param delta_t the time difference between current and previous timestamps
   * @returns all kinematics
   */

  function calculateAllKinematics(movement, delta, dt) {
    var dl = calculateDistance(delta);
    var alpha = dl === 0 ? 0 : 1 / dl;
    var beta = dt === 0 ? 0 : 1 / dt;
    var velocity = beta * dl;
    var velocities = delta.map(function (v) {
      return beta * v;
    });
    var direction = delta.map(function (v) {
      return alpha * v;
    });
    var distance = calculateDistance(movement);
    return {
      velocities: velocities,
      velocity: velocity,
      distance: distance,
      direction: direction
    };
  }
  /**
   * Because IE doesn't support `Math.sign` function, so we use the polyfill version of the function.
   * This polyfill function is suggested by Mozilla:
   * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/sign#Polyfill
   * @param x target number
   */

  function sign(x) {
    if (Math.sign) return Math.sign(x);
    return Number(x > 0) - Number(x < 0) || +x;
  }

  function minMax(value, min, max) {
    return Math.max(min, Math.min(value, max));
  } // Based on @aholachek ;)
  // https://twitter.com/chpwn/status/285540192096497664
  // iOS constant = 0.55
  // https://medium.com/@nathangitter/building-fluid-interfaces-ios-swift-9732bb934bf5


  function rubberband2(distance, constant) {
    // default constant from the article is 0.7
    return Math.pow(distance, constant * 5);
  }

  function rubberband(distance, dimension, constant) {
    if (dimension === 0 || Math.abs(dimension) === Infinity) return rubberband2(distance, constant);
    return distance * dimension * constant / (dimension + constant * distance);
  }

  function rubberbandIfOutOfBounds(position, min, max, constant) {
    if (constant === void 0) {
      constant = 0.15;
    }

    if (constant === 0) return minMax(position, min, max);
    if (position < min) return -rubberband(min - position, max - min, constant) + min;
    if (position > max) return +rubberband(position - max, max - min, constant) + max;
    return position;
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    subClass.__proto__ = superClass;
  }

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  function _assertThisInitialized(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return self;
  }

  function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
  }

  function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;

    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

    return arr2;
  }

  function _createForOfIteratorHelperLoose(o, allowArrayLike) {
    var it;

    if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
      if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
        if (it) o = it;
        var i = 0;
        return function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        };
      }

      throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
    }

    it = o[Symbol.iterator]();
    return it.next.bind(it);
  }

  function noop() {}
  /**
   * TODO Beware that only optimized cases are covered in tests =)
   * TODO Need to cover general case as well
   *
   * @param fns
   */

  function chainFns() {
    for (var _len = arguments.length, fns = new Array(_len), _key = 0; _key < _len; _key++) {
      fns[_key] = arguments[_key];
    }

    if (fns.length === 0) return noop;
    if (fns.length === 1) return fns[0];
    return function () {
      var result;

      for (var _iterator = _createForOfIteratorHelperLoose(fns), _step; !(_step = _iterator()).done;) {
        var fn = _step.value;
        result = fn.apply(this, arguments) || result;
      }

      return result;
    };
  }
  /**
   * Expects a simple value or 2D vector (an array with 2 elements) and
   * always returns 2D vector. If simple value is passed, returns a
   * vector with this value as both coordinates.
   *
   * @param value
   */

  function ensureVector(value, fallback) {
    if (value === undefined) {
      if (fallback === undefined) {
        throw new Error('Must define fallback value if undefined is expected');
      }

      value = fallback;
    }

    if (Array.isArray(value)) return value;
    return [value, value];
  }
  /**
   * Helper for defining a default value
   *
   * @param value
   * @param fallback
   */

  function assignDefault(value, fallback) {
    return Object.assign({}, fallback, value || {});
  }
  /**
   * Resolves getters (functions) by calling them
   * If simple value is given it just passes through
   *
   * @param v
   */

  function valueFn(v) {
    if (typeof v === 'function') {
      for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        args[_key2 - 1] = arguments[_key2];
      }

      // @ts-ignore
      return v.apply(void 0, args);
    } else {
      return v;
    }
  }

  function resolveWith(config, resolvers) {
    if (config === void 0) {
      config = {};
    }

    var result = {};

    for (var _i = 0, _Object$entries = Object.entries(resolvers); _i < _Object$entries.length; _i++) {
      var _Object$entries$_i = _Object$entries[_i],
          key = _Object$entries$_i[0],
          resolver = _Object$entries$_i[1];

      switch (typeof resolver) {
        case 'function':
          result[key] = resolver.call(result, config[key], key, config);
          break;

        case 'object':
          result[key] = resolveWith(config[key], resolver);
          break;

        case 'boolean':
          if (resolver) result[key] = config[key];
          break;
      }
    }

    return result;
  }

  var DEFAULT_DRAG_DELAY = 180;
  var DEFAULT_RUBBERBAND = 0.15;
  var DEFAULT_SWIPE_VELOCITY = 0.5;
  var DEFAULT_SWIPE_DISTANCE = 60;
  var InternalGestureOptionsNormalizers = {
    threshold: function threshold(value) {
      if (value === void 0) {
        value = 0;
      }

      return ensureVector(value);
    },
    rubberband: function rubberband(value) {
      if (value === void 0) {
        value = 0;
      }

      switch (value) {
        case true:
          return ensureVector(DEFAULT_RUBBERBAND);

        case false:
          return ensureVector(0);

        default:
          return ensureVector(value);
      }
    },
    enabled: function enabled(value) {
      if (value === void 0) {
        value = true;
      }

      return value;
    },
    triggerAllEvents: function triggerAllEvents(value) {
      if (value === void 0) {
        value = false;
      }

      return value;
    },
    initial: function initial(value) {
      if (value === void 0) {
        value = 0;
      }

      if (typeof value === 'function') return value;
      return ensureVector(value);
    }
  };

  var InternalCoordinatesOptionsNormalizers = /*#__PURE__*/_extends({}, InternalGestureOptionsNormalizers, {
    axis: true,
    lockDirection: function lockDirection(value) {
      if (value === void 0) {
        value = false;
      }

      return value;
    },
    bounds: function bounds(value) {
      if (value === void 0) {
        value = {};
      }

      if (typeof value === 'function') return function (state) {
        return InternalCoordinatesOptionsNormalizers.bounds(value(state));
      };
      var _value2 = value,
          _value2$left = _value2.left,
          left = _value2$left === void 0 ? -Infinity : _value2$left,
          _value2$right = _value2.right,
          right = _value2$right === void 0 ? Infinity : _value2$right,
          _value2$top = _value2.top,
          top = _value2$top === void 0 ? -Infinity : _value2$top,
          _value2$bottom = _value2.bottom,
          bottom = _value2$bottom === void 0 ? Infinity : _value2$bottom;
      return [[left, right], [top, bottom]];
    }
  });

  var isBrowser = typeof window !== 'undefined' && window.document && window.document.createElement;
  var InternalGenericOptionsNormalizers = {
    enabled: function enabled(value) {
      if (value === void 0) {
        value = true;
      }

      return value;
    },
    domTarget: true,
    window: /*#__PURE__*/function (_window) {
      function window(_x) {
        return _window.apply(this, arguments);
      }

      window.toString = function () {
        return _window.toString();
      };

      return window;
    }(function (value) {
      if (value === void 0) {
        value = isBrowser ? window : undefined;
      }

      return value;
    }),
    eventOptions: function eventOptions(_temp) {
      var _ref = _temp === void 0 ? {} : _temp,
          _ref$passive = _ref.passive,
          passive = _ref$passive === void 0 ? true : _ref$passive,
          _ref$capture = _ref.capture,
          capture = _ref$capture === void 0 ? false : _ref$capture;

      return {
        passive: passive,
        capture: capture
      };
    }
  };

  var InternalDistanceAngleOptionsNormalizers = /*#__PURE__*/_extends({}, InternalGestureOptionsNormalizers, {
    bounds: function bounds(_value, _key, _ref2) {
      var _ref2$distanceBounds = _ref2.distanceBounds,
          distanceBounds = _ref2$distanceBounds === void 0 ? {} : _ref2$distanceBounds,
          _ref2$angleBounds = _ref2.angleBounds,
          angleBounds = _ref2$angleBounds === void 0 ? {} : _ref2$angleBounds;

      var _distanceBounds = function _distanceBounds(state) {
        var D = assignDefault(valueFn(distanceBounds, state), {
          min: -Infinity,
          max: Infinity
        });
        return [D.min, D.max];
      };

      var _angleBounds = function _angleBounds(state) {
        var A = assignDefault(valueFn(angleBounds, state), {
          min: -Infinity,
          max: Infinity
        });
        return [A.min, A.max];
      };

      if (typeof distanceBounds !== 'function' && typeof angleBounds !== 'function') return [_distanceBounds(), _angleBounds()];
      return function (state) {
        return [_distanceBounds(state), _angleBounds(state)];
      };
    }
  });

  var InternalDragOptionsNormalizers = /*#__PURE__*/_extends({}, InternalCoordinatesOptionsNormalizers, {
    threshold: function threshold(v, _k, _ref3) {
      var _ref3$filterTaps = _ref3.filterTaps,
          filterTaps = _ref3$filterTaps === void 0 ? false : _ref3$filterTaps,
          _ref3$lockDirection = _ref3.lockDirection,
          lockDirection = _ref3$lockDirection === void 0 ? false : _ref3$lockDirection,
          _ref3$axis = _ref3.axis,
          axis = _ref3$axis === void 0 ? undefined : _ref3$axis;
      var A = ensureVector(v, filterTaps ? 3 : lockDirection ? 1 : axis ? 1 : 0);
      this.filterTaps = filterTaps || A[0] + A[1] > 0;
      return A;
    },
    swipeVelocity: function swipeVelocity(v) {
      if (v === void 0) {
        v = DEFAULT_SWIPE_VELOCITY;
      }

      return ensureVector(v);
    },
    swipeDistance: function swipeDistance(v) {
      if (v === void 0) {
        v = DEFAULT_SWIPE_DISTANCE;
      }

      return ensureVector(v);
    },
    delay: function delay(value) {
      if (value === void 0) {
        value = 0;
      }

      switch (value) {
        case true:
          return DEFAULT_DRAG_DELAY;

        case false:
          return 0;

        default:
          return value;
      }
    }
  });

  function getInternalGenericOptions(config) {
    if (config === void 0) {
      config = {};
    }

    // TODO warn when passive is set to true and domTarget is undefined
    return resolveWith(config, InternalGenericOptionsNormalizers);
  }
  function getInternalCoordinatesOptions(config) {
    if (config === void 0) {
      config = {};
    }

    return resolveWith(config, InternalCoordinatesOptionsNormalizers);
  }
  function getInternalDistanceAngleOptions(config) {
    if (config === void 0) {
      config = {};
    }

    return resolveWith(config, InternalDistanceAngleOptionsNormalizers);
  }
  function getInternalDragOptions(config) {
    if (config === void 0) {
      config = {};
    }

    return resolveWith(config, InternalDragOptionsNormalizers);
  }

  function _buildMoveConfig(_ref) {
    var domTarget = _ref.domTarget,
        eventOptions = _ref.eventOptions,
        window = _ref.window,
        enabled = _ref.enabled,
        rest = _objectWithoutPropertiesLoose(_ref, ["domTarget", "eventOptions", "window", "enabled"]);

    var opts = getInternalGenericOptions({
      domTarget: domTarget,
      eventOptions: eventOptions,
      window: window,
      enabled: enabled
    });
    opts.move = getInternalCoordinatesOptions(rest);
    return opts;
  }
  function _buildHoverConfig(_ref2) {
    var domTarget = _ref2.domTarget,
        eventOptions = _ref2.eventOptions,
        window = _ref2.window,
        enabled = _ref2.enabled,
        rest = _objectWithoutPropertiesLoose(_ref2, ["domTarget", "eventOptions", "window", "enabled"]);

    var opts = getInternalGenericOptions({
      domTarget: domTarget,
      eventOptions: eventOptions,
      window: window,
      enabled: enabled
    });
    opts.hover = _extends({
      enabled: true
    }, rest);
    return opts;
  }
  function _buildDragConfig(_ref3) {
    var domTarget = _ref3.domTarget,
        eventOptions = _ref3.eventOptions,
        window = _ref3.window,
        enabled = _ref3.enabled,
        rest = _objectWithoutPropertiesLoose(_ref3, ["domTarget", "eventOptions", "window", "enabled"]);

    var opts = getInternalGenericOptions({
      domTarget: domTarget,
      eventOptions: eventOptions,
      window: window,
      enabled: enabled
    });
    opts.drag = getInternalDragOptions(rest);
    return opts;
  }
  function _buildPinchConfig(_ref4) {
    var domTarget = _ref4.domTarget,
        eventOptions = _ref4.eventOptions,
        window = _ref4.window,
        enabled = _ref4.enabled,
        rest = _objectWithoutPropertiesLoose(_ref4, ["domTarget", "eventOptions", "window", "enabled"]);

    var opts = getInternalGenericOptions({
      domTarget: domTarget,
      eventOptions: eventOptions,
      window: window,
      enabled: enabled
    });
    opts.pinch = getInternalDistanceAngleOptions(rest);
    return opts;
  }
  function _buildScrollConfig(_ref5) {
    var domTarget = _ref5.domTarget,
        eventOptions = _ref5.eventOptions,
        window = _ref5.window,
        enabled = _ref5.enabled,
        rest = _objectWithoutPropertiesLoose(_ref5, ["domTarget", "eventOptions", "window", "enabled"]);

    var opts = getInternalGenericOptions({
      domTarget: domTarget,
      eventOptions: eventOptions,
      window: window,
      enabled: enabled
    });
    opts.scroll = getInternalCoordinatesOptions(rest);
    return opts;
  }
  function _buildWheelConfig(_ref6) {
    var domTarget = _ref6.domTarget,
        eventOptions = _ref6.eventOptions,
        window = _ref6.window,
        enabled = _ref6.enabled,
        rest = _objectWithoutPropertiesLoose(_ref6, ["domTarget", "eventOptions", "window", "enabled"]);

    var opts = getInternalGenericOptions({
      domTarget: domTarget,
      eventOptions: eventOptions,
      window: window,
      enabled: enabled
    });
    opts.wheel = getInternalCoordinatesOptions(rest);
    return opts;
  }
  function buildComplexConfig(config, actions) {
    if (config === void 0) {
      config = {};
    }

    if (actions === void 0) {
      actions = new Set();
    }

    var _config = config,
        drag = _config.drag,
        wheel = _config.wheel,
        move = _config.move,
        scroll = _config.scroll,
        pinch = _config.pinch,
        hover = _config.hover,
        eventOptions = _config.eventOptions,
        window = _config.window,
        domTarget = _config.domTarget,
        enabled = _config.enabled;
    var mergedConfig = getInternalGenericOptions({
      eventOptions: eventOptions,
      window: window,
      domTarget: domTarget,
      enabled: enabled
    });
    if (actions.has('onDrag')) mergedConfig.drag = getInternalDragOptions(drag);
    if (actions.has('onWheel')) mergedConfig.wheel = getInternalCoordinatesOptions(wheel);
    if (actions.has('onScroll')) mergedConfig.scroll = getInternalCoordinatesOptions(scroll);
    if (actions.has('onMove')) mergedConfig.move = getInternalCoordinatesOptions(move);
    if (actions.has('onPinch')) mergedConfig.pinch = getInternalDistanceAngleOptions(pinch);
    if (actions.has('onHover')) mergedConfig.hover = _extends({
      enabled: true
    }, hover);
    return mergedConfig;
  }

  function getInitial(mixed) {
    return _extends({
      _active: false,
      _blocked: false,
      _intentional: [false, false],
      _movement: [0, 0],
      _initial: [0, 0],
      _bounds: [[-Infinity, Infinity], [-Infinity, Infinity]],
      _lastEventType: undefined,
      event: undefined,
      // currentTarget: undefined,
      // pointerId: undefined,
      intentional: false,
      values: [0, 0],
      velocities: [0, 0],
      delta: [0, 0],
      movement: [0, 0],
      offset: [0, 0],
      lastOffset: [0, 0],
      direction: [0, 0],
      initial: [0, 0],
      previous: [0, 0],
      first: false,
      last: false,
      active: false,
      timeStamp: 0,
      startTime: 0,
      elapsedTime: 0,
      cancel: noop,
      canceled: false,
      memo: undefined,
      args: undefined
    }, mixed);
  }

  function getInitialState() {
    var shared = {
      hovering: false,
      scrolling: false,
      wheeling: false,
      dragging: false,
      moving: false,
      pinching: false,
      touches: 0,
      buttons: 0,
      down: false,
      shiftKey: false,
      altKey: false,
      metaKey: false,
      ctrlKey: false
    };
    var drag = getInitial({
      axis: undefined,
      xy: [0, 0],
      vxvy: [0, 0],
      velocity: 0,
      distance: 0,
      _isTap: true,
      _delayedEvent: false,
      _pointerId: undefined,
      tap: false,
      swipe: [0, 0]
    });
    var pinch = getInitial({
      da: [0, 0],
      vdva: [0, 0],
      // @ts-ignore origin can never be passed as undefined in userland
      origin: undefined,
      turns: 0
    });
    var wheel = getInitial({
      axis: undefined,
      xy: [0, 0],
      vxvy: [0, 0],
      velocity: 0,
      distance: 0
    });
    var move = getInitial({
      axis: undefined,
      xy: [0, 0],
      vxvy: [0, 0],
      velocity: 0,
      distance: 0
    });
    var scroll = getInitial({
      axis: undefined,
      xy: [0, 0],
      vxvy: [0, 0],
      velocity: 0,
      distance: 0
    });
    return {
      shared: shared,
      drag: drag,
      pinch: pinch,
      wheel: wheel,
      move: move,
      scroll: scroll
    };
  }

  var RecognizersMap = /*#__PURE__*/new Map();
  /**
   * @private
   * Recognizer abstract class.
   */

  var Recognizer = /*#__PURE__*/function () {
    /**
     * Creates an instance of a gesture recognizer.
     * @param stateKey drag, move, pinch, etc.
     * @param controller the controller attached to the gesture
     * @param [args] the args that should be passed to the gesture handler
     */
    function Recognizer(controller, args) {
      var _this = this;

      if (args === void 0) {
        args = [];
      }

      this.controller = controller;
      this.args = args;
      this.debounced = true; // Convenience method to set a timeout for a given gesture

      this.setTimeout = function (callback, ms) {
        var _window;

        if (ms === void 0) {
          ms = 140;
        }

        clearTimeout(_this.controller.timeouts[_this.stateKey]);

        for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
          args[_key - 2] = arguments[_key];
        }

        _this.controller.timeouts[_this.stateKey] = (_window = window).setTimeout.apply(_window, [callback, ms].concat(args));
      }; // Convenience method to clear a timeout for a given gesture


      this.clearTimeout = function () {
        clearTimeout(_this.controller.timeouts[_this.stateKey]);
      };
      /**
       * Fires the gesture handler
       */


      this.fireGestureHandler = function (forceFlag) {
        if (forceFlag === void 0) {
          forceFlag = false;
        }

        /**
         * If the gesture has been blocked (this can happen when the gesture has started in an unwanted direction),
         * clean everything and don't do anything.
         */
        if (_this.state._blocked) {
          // we need debounced gestures to end by themselves
          if (!_this.debounced) {
            _this.state._active = false;

            _this.clean();
          }

          return null;
        } // If the gesture has no intentional dimension, don't fire the handler.


        if (!forceFlag && !_this.state.intentional && !_this.config.triggerAllEvents) return null;

        if (_this.state.intentional) {
          var prev_active = _this.state.active;
          var next_active = _this.state._active;
          _this.state.active = next_active;
          _this.state.first = next_active && !prev_active;
          _this.state.last = prev_active && !next_active;
          _this.controller.state.shared[_this.ingKey] = next_active; // Sets dragging, pinching, etc. to the gesture active state
        }

        var state = _extends({}, _this.controller.state.shared, _this.state, _this.mapStateValues(_this.state)); // @ts-ignore


        var newMemo = _this.handler(state); // Sets memo to the returned value of the handler (unless it's not undefined)


        _this.state.memo = newMemo !== void 0 ? newMemo : _this.state.memo; // Cleans the gesture when the gesture is no longer active.

        if (!_this.state._active) _this.clean();
        return state;
      };
    } // Returns the gesture config


    var _proto = Recognizer.prototype;

    // Convenience method to update the shared state
    _proto.updateSharedState = function updateSharedState(sharedState) {
      Object.assign(this.controller.state.shared, sharedState);
    } // Convenience method to update the gesture state
    ;

    _proto.updateGestureState = function updateGestureState(gestureState) {
      Object.assign(this.state, gestureState);
    }
    /**
     * Returns state properties depending on the movement and state.
     *
     * Should be overriden for custom behavior, doesn't do anything in the implementation
     * below.
     */
    ;

    _proto.checkIntentionality = function checkIntentionality(_intentional, _movement) {
      return {
        _intentional: _intentional,
        _blocked: false
      };
    }
    /**
     * Returns basic movement properties for the gesture based on the next values and current state.
     */
    ;

    _proto.getMovement = function getMovement(values) {
      var _this$config = this.config,
          initial = _this$config.initial,
          bounds = _this$config.bounds,
          rubberband = _this$config.rubberband,
          T = _this$config.threshold;
      var _this$state = this.state,
          _bounds = _this$state._bounds,
          _initial = _this$state._initial,
          _active = _this$state._active,
          wasIntentional = _this$state._intentional,
          lastOffset = _this$state.lastOffset,
          prevMovement = _this$state.movement;
      var M = this.getInternalMovement(values, this.state);
      var i0 = wasIntentional[0] === false ? getIntentionalDisplacement(M[0], T[0]) : wasIntentional[0];
      var i1 = wasIntentional[1] === false ? getIntentionalDisplacement(M[1], T[1]) : wasIntentional[1]; // Get gesture specific state properties based on intentionality and movement.

      var intentionalityCheck = this.checkIntentionality([i0, i1], M);

      if (intentionalityCheck._blocked) {
        return _extends({}, intentionalityCheck, {
          _movement: M,
          delta: [0, 0]
        });
      }

      var _intentional = intentionalityCheck._intentional;
      var _movement = M;

      var __cachedBounds;

      var __cachedInitial;

      if (_intentional[0] !== false && wasIntentional[0] === false) {
        __cachedInitial = valueFn(initial, this.state);
        __cachedBounds = valueFn(bounds, this.state);
        _initial[0] = __cachedInitial[0];
        _bounds[0] = __cachedBounds[0];
      }

      if (_intentional[1] !== false && wasIntentional[1] === false) {
        var _cachedInitial, _cachedBounds;

        __cachedInitial = (_cachedInitial = __cachedInitial) !== null && _cachedInitial !== void 0 ? _cachedInitial : valueFn(initial, this.state);
        __cachedBounds = (_cachedBounds = __cachedBounds) !== null && _cachedBounds !== void 0 ? _cachedBounds : valueFn(bounds, this.state);
        _initial[1] = __cachedInitial[1];
        _bounds[1] = __cachedBounds[1];
      }
      /**
       * The movement sent to the handler has 0 in its dimensions when intentionality is false.
       * It is calculated from the actual movement minus the threshold.
       */


      var movement = [_intentional[0] !== false ? M[0] - _intentional[0] : _initial[0], _intentional[1] !== false ? M[1] - _intentional[1] : _initial[1]];
      var offset = addV(movement, lastOffset);
      /**
       * Rubberband should be 0 when the gesture is no longer active, so that movement
       * and offset can return within their bounds.
       */

      var _rubberband = _active ? rubberband : [0, 0];

      movement = computeRubberband(_bounds, addV(movement, _initial), _rubberband);
      return _extends({}, intentionalityCheck, {
        intentional: _intentional[0] !== false || _intentional[1] !== false,
        _initial: _initial,
        _movement: _movement,
        movement: movement,
        values: values,
        offset: computeRubberband(_bounds, offset, _rubberband),
        delta: subV(movement, prevMovement)
      });
    } // Cleans the gesture. Can be overriden by gestures.
    ;

    _proto.clean = function clean() {
      this.clearTimeout();
    };

    _createClass(Recognizer, [{
      key: "config",
      get: function get() {
        return this.controller.config[this.stateKey];
      } // Is the gesture enabled

    }, {
      key: "enabled",
      get: function get() {
        return this.controller.config.enabled && this.config.enabled;
      } // Returns the controller state for a given gesture

    }, {
      key: "state",
      get: function get() {
        return this.controller.state[this.stateKey];
      } // Returns the gesture handler

    }, {
      key: "handler",
      get: function get() {
        return this.controller.handlers[this.stateKey];
      }
    }]);

    return Recognizer;
  }(); //--------------------------------------------

  function getIntentionalDisplacement(movement, threshold) {
    if (Math.abs(movement) >= threshold) {
      return sign(movement) * threshold;
    } else {
      return false;
    }
  }

  function computeRubberband(bounds, _ref, _ref2) {
    var Vx = _ref[0],
        Vy = _ref[1];
    var Rx = _ref2[0],
        Ry = _ref2[1];
    var _bounds$ = bounds[0],
        X1 = _bounds$[0],
        X2 = _bounds$[1],
        _bounds$2 = bounds[1],
        Y1 = _bounds$2[0],
        Y2 = _bounds$2[1];
    return [rubberbandIfOutOfBounds(Vx, X1, X2, Rx), rubberbandIfOutOfBounds(Vy, Y1, Y2, Ry)];
  }
  /**
   * Returns a generic, common payload for all gestures from an event.
   */


  function getGenericPayload(_ref3, event, isStartEvent) {
    var state = _ref3.state,
        args = _ref3.args;
    var timeStamp = event.timeStamp,
        _lastEventType = event.type;
    var previous = state.values;
    var elapsedTime = isStartEvent ? 0 : timeStamp - state.startTime;
    return {
      _lastEventType: _lastEventType,
      event: event,
      timeStamp: timeStamp,
      elapsedTime: elapsedTime,
      args: args,
      previous: previous
    };
  }
  /**
   * Returns the reinitialized start state for the gesture.
   * Should be common to all gestures.
   */

  function getStartGestureState(recognizer, values, event) {
    var offset = recognizer.state.offset;
    var startTime = event.timeStamp;
    return _extends({}, getInitialState()[recognizer.stateKey], {
      _active: true,
      values: values,
      initial: values,
      offset: offset,
      lastOffset: offset,
      startTime: startTime
    });
  }

  function partial(func, state) {
    return function (event) {
      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      // @ts-ignore
      return func.call.apply(func, [this, _extends({}, state, {
        event: event
      })].concat(args));
    };
  }
  /**
   * The controller will keep track of the state for all gestures and also keep
   * track of timeouts, and window listeners.
   */


  var Controller = function Controller(classes) {
    var _this = this;

    this.classes = classes;

    this.bind = function () {
      var bindings = {};

      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      for (var _iterator = _createForOfIteratorHelperLoose(_this.classes), _step; !(_step = _iterator()).done;) {
        var RecognizerClass = _step.value;
        new RecognizerClass(_this, args).addBindings(bindings);
      } // we also add event bindings for native handlers


      for (var _i = 0, _Object$entries = Object.entries(_this.nativeRefs); _i < _Object$entries.length; _i++) {
        var _Object$entries$_i = _Object$entries[_i],
            event = _Object$entries$_i[0],
            handler = _Object$entries$_i[1];
        addBindings(bindings, event, partial(handler, _extends({}, _this.state.shared, {
          args: args
        })));
      }

      if (_this.config.domTarget) {
        // If config.domTarget is set we add event listeners to it and return the clean function.
        return updateDomListeners(_this, bindings);
      } else {
        // If not, we return an object that contains gesture handlers mapped to react handler event keys.
        return getPropsListener(_this, bindings);
      }
    };

    this.effect = function () {
      if (_this.config.domTarget) _this.bind();
      return _this.clean;
    };
    /**
     * Function ran on component unmount: cleans timeouts and removes dom listeners set by the bind function.
     */


    this.clean = function () {
      var domTarget = getDomTargetFromConfig(_this.config);
      var eventOptions = _this.config.eventOptions;
      if (domTarget) removeListeners(domTarget, takeAll(_this.domListeners), eventOptions);
      Object.values(_this.timeouts).forEach(clearTimeout);
      clearAllWindowListeners(_this);
    };

    this.state = getInitialState();
    this.timeouts = {};
    this.domListeners = [];
    this.windowListeners = {};
  };
  function clearAllWindowListeners(controller) {
    var _controller$config = controller.config,
        el = _controller$config.window,
        eventOptions = _controller$config.eventOptions,
        windowListeners = controller.windowListeners;
    if (!el) return;

    for (var stateKey in windowListeners) {
      var handlers = windowListeners[stateKey];
      removeListeners(el, handlers, eventOptions);
    }

    controller.windowListeners = {};
  }
  function clearWindowListeners(_ref, stateKey) {
    var config = _ref.config,
        windowListeners = _ref.windowListeners;
    if (!config.window) return;
    removeListeners(config.window, windowListeners[stateKey], config.eventOptions);
    delete windowListeners[stateKey];
  }
  function updateWindowListeners(_ref2, stateKey, listeners) {
    var config = _ref2.config,
        windowListeners = _ref2.windowListeners;

    if (listeners === void 0) {
      listeners = [];
    }

    if (!config.window) return;
    removeListeners(config.window, windowListeners[stateKey], config.eventOptions);
    addListeners(config.window, windowListeners[stateKey] = listeners, config.eventOptions);
  }

  function updateDomListeners(_ref3, bindings) {
    var config = _ref3.config,
        domListeners = _ref3.domListeners;
    var domTarget = getDomTargetFromConfig(config);
    if (!domTarget) throw new Error('domTarget must be defined');
    var eventOptions = config.eventOptions;
    removeListeners(domTarget, takeAll(domListeners), eventOptions);

    for (var _i2 = 0, _Object$entries2 = Object.entries(bindings); _i2 < _Object$entries2.length; _i2++) {
      var _Object$entries2$_i = _Object$entries2[_i2],
          key = _Object$entries2$_i[0],
          fns = _Object$entries2$_i[1];
      var name = key.slice(2).toLowerCase();
      domListeners.push([name, chainFns.apply(void 0, fns)]);
    }

    addListeners(domTarget, domListeners, eventOptions);
  }

  function getPropsListener(_ref4, bindings) {
    var config = _ref4.config;
    var props = {};
    var captureString = config.eventOptions.capture ? 'Capture' : '';

    for (var _i3 = 0, _Object$entries3 = Object.entries(bindings); _i3 < _Object$entries3.length; _i3++) {
      var _Object$entries3$_i = _Object$entries3[_i3],
          event = _Object$entries3$_i[0],
          fns = _Object$entries3$_i[1];
      var fnsArray = Array.isArray(fns) ? fns : [fns];
      var key = event + captureString;
      props[key] = chainFns.apply(void 0, fnsArray);
    }

    return props;
  }

  function takeAll(array) {
    if (array === void 0) {
      array = [];
    }

    return array.splice(0, array.length);
  }

  function getDomTargetFromConfig(_ref5) {
    var domTarget = _ref5.domTarget;
    return domTarget && 'value' in domTarget ? domTarget.value : domTarget;
  }
  /**
   * bindings is an object which keys match ReactEventHandlerKeys.
   * Since a recognizer might want to bind a handler function to an event key already used by a previously
   * added recognizer, we need to make sure that each event key is an array of all the functions mapped for
   * that key.
   */


  function addBindings(bindings, name, fn) {
    if (!bindings[name]) bindings[name] = [];
    bindings[name].push(fn);
  }

  function addListeners(el, listeners, options) {
    if (listeners === void 0) {
      listeners = [];
    }

    if (options === void 0) {
      options = {};
    }

    for (var _iterator2 = _createForOfIteratorHelperLoose(listeners), _step2; !(_step2 = _iterator2()).done;) {
      var _step2$value = _step2.value,
          eventName = _step2$value[0],
          eventHandler = _step2$value[1];
      el.addEventListener(eventName, eventHandler, options);
    }
  }

  function removeListeners(el, listeners, options) {
    if (listeners === void 0) {
      listeners = [];
    }

    if (options === void 0) {
      options = {};
    }

    for (var _iterator3 = _createForOfIteratorHelperLoose(listeners), _step3; !(_step3 = _iterator3()).done;) {
      var _step3$value = _step3.value,
          eventName = _step3$value[0],
          eventHandler = _step3$value[1];
      el.removeEventListener(eventName, eventHandler, options);
    }
  }

  /**
   * Utility hook called by all gesture hooks and that will be responsible for the internals.
   *
   * @param handlers
   * @param classes
   * @param config
   * @param nativeHandlers - native handlers such as onClick, onMouseDown, etc.
   */

  function useRecognizers(handlers, config, nativeHandlers) {
    if (nativeHandlers === void 0) {
      nativeHandlers = {};
    }

    var classes = resolveClasses(handlers); // const controller = React.useMemo(() => new Controller(classes), []);

    var controller = new Controller(classes);
    controller.config = config;
    controller.handlers = handlers;
    controller.nativeRefs = nativeHandlers; // React.useEffect(controller.effect, []);
    // console.log(controller.effect);//

    vueDemi.onMounted(controller.effect); // @ts-ignore

    if (controller.config.domTarget) return deprecationNoticeForDomTarget; // @ts-ignore

    return controller.bind;
  }

  function deprecationNoticeForDomTarget() {
    {
      console.warn("Deprecation notice: When the `domTarget` option is specified, you don't need to write `useEffect(bind, [bind])` anymore: event binding is now made handled internally to this lib.\n\nNext version won't return anything when `domTarget` is provided, therefore your code will break if you try to call `useEffect`.");
    }
  }

  function resolveClasses(internalHandlers) {
    var classes = new Set();
    if (internalHandlers.drag) classes.add(RecognizersMap.get('drag'));
    if (internalHandlers.wheel) classes.add(RecognizersMap.get('wheel'));
    if (internalHandlers.scroll) classes.add(RecognizersMap.get('scroll'));
    if (internalHandlers.move) classes.add(RecognizersMap.get('move'));
    if (internalHandlers.pinch) classes.add(RecognizersMap.get('pinch'));
    if (internalHandlers.hover) classes.add(RecognizersMap.get('hover'));
    return classes;
  }

  /**
   * @private
   * Abstract class for coordinates-based gesture recongizers
   */

  var CoordinatesRecognizer = /*#__PURE__*/function (_Recognizer) {
    _inheritsLoose(CoordinatesRecognizer, _Recognizer);

    function CoordinatesRecognizer() {
      return _Recognizer.apply(this, arguments) || this;
    }

    var _proto = CoordinatesRecognizer.prototype;

    /**
     * Returns the real movement (without taking intentionality into acount)
     */
    _proto.getInternalMovement = function getInternalMovement(values, state) {
      return subV(values, state.initial);
    }
    /**
     * In coordinates-based gesture, this function will detect the first intentional axis,
     * lock the gesture axis if lockDirection is specified in the config, block the gesture
     * if the first intentional axis doesn't match the specified axis in config.
     */
    ;

    _proto.checkIntentionality = function checkIntentionality(_intentional, _movement) {
      if (_intentional[0] === false && _intentional[1] === false) {
        return {
          _intentional: _intentional,
          axis: this.state.axis
        };
      }

      var _movement$map = _movement.map(Math.abs),
          absX = _movement$map[0],
          absY = _movement$map[1];

      var axis = this.state.axis || (absX > absY ? 'x' : absX < absY ? 'y' : undefined);
      if (!this.config.axis && !this.config.lockDirection) return {
        _intentional: _intentional,
        _blocked: false,
        axis: axis
      };
      if (!axis) return {
        _intentional: [false, false],
        _blocked: false,
        axis: axis
      };
      if (!!this.config.axis && axis !== this.config.axis) return {
        _intentional: _intentional,
        _blocked: true,
        axis: axis
      };
      _intentional[axis === 'x' ? 1 : 0] = false;
      return {
        _intentional: _intentional,
        _blocked: false,
        axis: axis
      };
    };

    _proto.getKinematics = function getKinematics(values, event) {
      var state = this.getMovement(values);

      if (!state._blocked) {
        var dt = event.timeStamp - this.state.timeStamp;
        Object.assign(state, calculateAllKinematics(state.movement, state.delta, dt));
      }

      return state;
    };

    _proto.mapStateValues = function mapStateValues(state) {
      return {
        xy: state.values,
        vxvy: state.velocities
      };
    };

    return CoordinatesRecognizer;
  }(Recognizer);

  var WEBKIT_DISTANCE_SCALE_FACTOR = 260;
  /**
   * Whether the browser supports GestureEvent (ie Safari)
   * @returns true if the browser supports gesture event
   */

  function supportsGestureEvents() {
    try {
      // TODO [TS] possibly find GestureEvent definitions?
      // @ts-ignore: no type definitions for webkit GestureEvents
      return 'constructor' in GestureEvent;
    } catch (e) {
      return false;
    }
  }
  function supportsTouchEvents() {
    return typeof window !== 'undefined' && window.ontouchstart === null;
  }

  function getTouchEvents(event) {
    if ('touches' in event) {
      var targetTouches = event.targetTouches,
          changedTouches = event.changedTouches;
      return targetTouches.length > 0 ? targetTouches : changedTouches;
    }

    return null;
  }

  function getGenericEventData(event) {
    var buttons = 'buttons' in event ? event.buttons : 0;
    var touchEvents = getTouchEvents(event);
    var touches = touchEvents && touchEvents.length || 0;
    var down = touches > 0 || buttons > 0;
    var shiftKey = event.shiftKey,
        altKey = event.altKey,
        metaKey = event.metaKey,
        ctrlKey = event.ctrlKey; // TODO check if this might create some overrides?

    return {
      touches: touches,
      down: down,
      buttons: buttons,
      shiftKey: shiftKey,
      altKey: altKey,
      metaKey: metaKey,
      ctrlKey: ctrlKey
    };
  }
  /**
   * Gets pointer event values.
   * @param event
   * @returns pointer event values
   */

  function getPointerEventValues(event) {
    var touchEvents = getTouchEvents(event);

    var _ref = touchEvents ? touchEvents[0] : event,
        clientX = _ref.clientX,
        clientY = _ref.clientY;

    return [clientX, clientY];
  }
  /**
   * Gets scroll event values
   * @param event
   * @returns scroll event values
   */

  function getScrollEventValues(event) {
    // If the currentTarget is the window then we return the scrollX/Y position.
    // If not (ie the currentTarget is a DOM element), then we return scrollLeft/Top
    var _event$currentTarget = event.currentTarget,
        scrollX = _event$currentTarget.scrollX,
        scrollY = _event$currentTarget.scrollY,
        scrollLeft = _event$currentTarget.scrollLeft,
        scrollTop = _event$currentTarget.scrollTop;
    return [scrollX || scrollLeft || 0, scrollY || scrollTop || 0];
  }
  /**
   * Gets wheel event values.
   * @param event
   * @returns wheel event values
   */

  function getWheelEventValues(event) {
    var deltaX = event.deltaX,
        deltaY = event.deltaY; //TODO implement polyfill ?
    // https://developer.mozilla.org/en-US/docs/Web/Events/wheel#Polyfill

    return [deltaX, deltaY];
  }
  /**
   * Gets webkit gesture event values.
   * @param event
   * @returns webkit gesture event values
   */

  function getWebkitGestureEventValues(event) {
    return [event.scale * WEBKIT_DISTANCE_SCALE_FACTOR, event.rotation];
  }
  /**
   * Gets two touches event data
   * @param event
   * @returns two touches event data
   */

  function getTwoTouchesEventData(event) {
    var _e$rotation;

    var targetTouches = event.targetTouches;
    var A = targetTouches[0],
        B = targetTouches[1];
    var dx = B.clientX - A.clientX;
    var dy = B.clientY - A.clientY;
    var cx = (B.clientX + A.clientX) / 2;
    var cy = (B.clientY + A.clientY) / 2;
    var e = 'nativeEvent' in event ? event['nativeEvent'] : event; //TODO: why never?

    var distance = Math.hypot(dx, dy);
    var angle = (_e$rotation = e.rotation) !== null && _e$rotation !== void 0 ? _e$rotation : -(Math.atan2(dx, dy) * 180) / Math.PI;
    var values = [distance, angle];
    var origin = [cx, cy];
    return {
      values: values,
      origin: origin
    };
  }

  var TAP_DISTANCE_THRESHOLD = 3;
  var SWIPE_MAX_ELAPSED_TIME = 220;
  var DragRecognizer = /*#__PURE__*/function (_CoordinatesRecognize) {
    _inheritsLoose(DragRecognizer, _CoordinatesRecognize);

    function DragRecognizer() {
      var _this;

      _this = _CoordinatesRecognize.apply(this, arguments) || this;
      _this.ingKey = 'dragging';
      _this.stateKey = 'drag';
      /**
       * TODO add back when setPointerCapture is widely wupported
       * https://caniuse.com/#search=setPointerCapture
       * private setPointers = (event: UseGestureEvent<PointerEvent>) => {
       *   const { currentTarget, pointerId } = event
       *   if (currentTarget) currentTarget.setPointerCapture(pointerId)
       *   this.updateGestureState({ currentTarget, pointerId })
       * }
              * private removePointers = () => {
       *   const { currentTarget, pointerId } = this.state
       *   if (currentTarget && pointerId) currentTarget.releasePointerCapture(pointerId)
       * }
       */

      _this.onDragStart = function (event) {
        if (!_this.enabled || _this.state._active) return;
        /**
         * TODO add back when setPointerCapture is widely supported
         * this.setPointers(event as PointerEvent)
         */

        updateWindowListeners(_this.controller, _this.stateKey, [['pointermove', _this.onDragChange], ['pointerup', _this.onDragEnd], ['pointercancel', _this.onDragEnd]]); // We set the state pointerId to the event.pointerId so we can make sure
        // that we lock the drag to the event initiating the gesture

        _this.updateGestureState({
          _pointerId: event.pointerId
        });

        if (_this.config.delay > 0) {
          _this.state._delayedEvent = true; // If it's a React SyntheticEvent we need to persist it so that we can use it async
          // if ('persist' in event && typeof event.persist === 'function') event.persist();
          // this.setTimeout(this.startDrag.bind(this), this.config.delay, event);
        } else {
          _this.startDrag(event);
        }
      };

      _this.onDragChange = function (event) {
        // If the gesture was canceled don't respond to the event.
        if (_this.state.canceled) return; // If the event pointerId doesn't match the initiating pointerId
        // don't respond to the event.

        if (event.pointerId !== _this.state._pointerId) return; // If the gesture isn't active then respond to the event only if
        // it's been delayed via the `delay` option, in which case start
        // the gesture immediately.

        if (!_this.state._active) {
          if (_this.state._delayedEvent) {
            _this.clearTimeout();

            _this.startDrag(event);
          }

          return;
        }

        var genericEventData = getGenericEventData(event); // If the event doesn't have any button / touches left we should cancel
        // the gesture. This may happen if the drag release happens outside the browser
        // window.

        if (!genericEventData.down) {
          _this.onDragEnd(event);

          return;
        }

        _this.updateSharedState(genericEventData);

        var values = getPointerEventValues(event);

        var kinematics = _this.getKinematics(values, event);

        var genericPayload = getGenericPayload(_assertThisInitialized(_this), event); // This verifies if the drag can be assimilated to a tap by checking
        // if the real distance of the drag (ie not accounting for the threshold) is
        // greater than the TAP_DISTANCE_THRESHOLD.

        var _isTap = _this.state._isTap;
        var realDistance = calculateDistance(kinematics._movement);
        if (_isTap && realDistance >= TAP_DISTANCE_THRESHOLD) _isTap = false;

        _this.updateGestureState(_extends({}, genericPayload, kinematics, {
          _isTap: _isTap
        }));

        _this.fireGestureHandler();
      };

      _this.onDragEnd = function (event) {
        // If the event pointerId doesn't match the initiating pointerId
        // don't respond to the event.
        if (event.pointerId !== _this.state._pointerId) return;
        _this.state._active = false;

        _this.updateSharedState({
          down: false,
          buttons: 0,
          touches: 0
        });

        var tap = _this.state._isTap;
        var _this$state$velocitie = _this.state.velocities,
            vx = _this$state$velocitie[0],
            vy = _this$state$velocitie[1];
        var _this$state$movement = _this.state.movement,
            mx = _this$state$movement[0],
            my = _this$state$movement[1];
        var _this$state$_intentio = _this.state._intentional,
            ix = _this$state$_intentio[0],
            iy = _this$state$_intentio[1];
        var _this$config$swipeVel = _this.config.swipeVelocity,
            svx = _this$config$swipeVel[0],
            svy = _this$config$swipeVel[1];
        var _this$config$swipeDis = _this.config.swipeDistance,
            sx = _this$config$swipeDis[0],
            sy = _this$config$swipeDis[1];

        var endState = _extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getMovement(_this.state.values));

        var swipe = [0, 0];

        if (endState.elapsedTime < SWIPE_MAX_ELAPSED_TIME) {
          if (ix !== false && Math.abs(vx) > svx && Math.abs(mx) > sx) swipe[0] = sign(vx);
          if (iy !== false && Math.abs(vy) > svy && Math.abs(my) > sy) swipe[1] = sign(vy);
        }

        _this.updateGestureState(_extends({}, endState, {
          tap: tap,
          swipe: swipe
        }));

        _this.fireGestureHandler(tap === true);
      };

      _this.clean = function () {
        _CoordinatesRecognize.prototype.clean.call(_assertThisInitialized(_this));

        _this.state._delayedEvent = false; // can't remember if this is useful?

        clearWindowListeners(_this.controller, _this.stateKey); // TODO add back when setPointerCapture is widely wupported
        // this.removePointers()
      };

      _this.onCancel = function () {
        if (_this.state.canceled) return;

        _this.updateGestureState({
          canceled: true
        });

        _this.state._active = false;

        _this.updateSharedState({
          down: false,
          buttons: 0,
          touches: 0
        });

        requestAnimationFrame(function () {
          return _this.fireGestureHandler();
        });
      };

      _this.onClick = function (event) {
        if (!_this.state._isTap) event.stopPropagation();
      };

      return _this;
    }

    var _proto = DragRecognizer.prototype;

    _proto.startDrag = function startDrag(event) {
      var values = getPointerEventValues(event);
      this.updateSharedState(getGenericEventData(event));
      this.updateGestureState(_extends({}, getStartGestureState(this, values, event), getGenericPayload(this, event, true), {
        _pointerId: event.pointerId,
        cancel: this.onCancel
      }));
      this.updateGestureState(this.getMovement(values));
      this.fireGestureHandler();
    };

    _proto.addBindings = function addBindings$1(bindings) {
      addBindings(bindings, 'onPointerDown', this.onDragStart);

      if (this.config.filterTaps) {
        var handler = this.controller.config.eventOptions.capture ? 'onClick' : 'onClickCapture';

        addBindings(bindings, handler, this.onClick);
      } // TODO add back when setPointerCapture is widely wupported
      // addBindings(bindings, 'onPointerMove', this.onDragChange)
      // addBindings(bindings, 'onPointerUp', this.onDragEnd)
      // addBindings(bindings, 'onPointerCancel', this.onDragEnd)

    };

    return DragRecognizer;
  }(CoordinatesRecognizer);

  /**
   * Inlined from https://github.com/alexreardon/memoize-one
   */
  function memoizeOne(resultFn, isEqual) {
    var lastThis;
    var lastArgs = [];
    var lastResult;
    var calledOnce = false;

    function memoized() {
      for (var _len = arguments.length, newArgs = new Array(_len), _key = 0; _key < _len; _key++) {
        newArgs[_key] = arguments[_key];
      }

      if (calledOnce && lastThis === this && isEqual(newArgs, lastArgs)) {
        return lastResult;
      }

      lastResult = resultFn.apply(this, newArgs);
      calledOnce = true;
      lastThis = this;
      lastArgs = newArgs;
      return lastResult;
    }

    return memoized;
  }

  /**
   * Taken from https://github.com/FormidableLabs/react-fast-compare
   *
   * Dropped comments and ArrayBuffer handling
   */
  function equal(a, b) {
    if (a === b) return true;

    if (a && b && typeof a == 'object' && typeof b == 'object') {
      if (a.constructor !== b.constructor) return false;
      var length, i, keys;

      if (Array.isArray(a)) {
        length = a.length;
        if (length !== b.length) return false;

        for (i = length; i-- !== 0;) {
          if (!equal(a[i], b[i])) return false;
        }

        return true;
      }

      var it;

      if (typeof Map === 'function' && a instanceof Map && b instanceof Map) {
        if (a.size !== b.size) return false;
        it = a.entries();

        while (!(i = it.next()).done) {
          if (!b.has(i.value[0])) return false;
        }

        it = a.entries();

        while (!(i = it.next()).done) {
          if (!equal(i.value[1], b.get(i.value[0]))) return false;
        }

        return true;
      }

      if (typeof Set === 'function' && a instanceof Set && b instanceof Set) {
        if (a.size !== b.size) return false;
        it = a.entries();

        while (!(i = it.next()).done) {
          if (!b.has(i.value[0])) return false;
        }

        return true;
      }

      if (a.constructor === RegExp) return a.source === b.source && a.flags === b.flags;
      if (a.valueOf !== Object.prototype.valueOf) return a.valueOf() === b.valueOf();
      if (a.toString !== Object.prototype.toString) return a.toString() === b.toString();
      keys = Object.keys(a);
      length = keys.length;
      if (length !== Object.keys(b).length) return false;

      for (i = length; i-- !== 0;) {
        if (!Object.prototype.hasOwnProperty.call(b, keys[i])) return false;
      }

      if (typeof Element !== 'undefined' && a instanceof Element) return false;

      for (i = length; i-- !== 0;) {
        if (keys[i] === '_owner' && a.$$typeof) continue;
        if (!equal(a[keys[i]], b[keys[i]])) return false;
      }

      return true;
    } // true if both NaN, false otherwise — NaN !== NaN → true
    // eslint-disable-next-line no-self-compare


    return a !== a && b !== b;
  }

  function isEqual(a, b) {
    try {
      return equal(a, b);
    } catch (error) {
      if ((error.message || '').match(/stack|recursion/i)) {
        console.warn('react-fast-compare cannot handle circular refs');
        return false;
      }

      throw error;
    }
  }

  /**
   * Drag hook.
   *
   * @param handler - the function fired every time the drag gesture updates
   * @param [config={}] - the config object including generic options and drag options
   */

  function useDrag(handler, config) {
    if (config === void 0) {
      config = {};
    }

    RecognizersMap.set('drag', DragRecognizer);
    var buildDragConfig = vueDemi.ref();

    if (!buildDragConfig.value) {
      buildDragConfig.value = memoizeOne(_buildDragConfig, isEqual);
    }

    return useRecognizers({
      drag: handler
    }, buildDragConfig.value(config));
  }

  /**
   * @private
   * Abstract class for distance/angle-based gesture recongizers
   */

  var DistanceAngleRecognizer = /*#__PURE__*/function (_Recognizer) {
    _inheritsLoose(DistanceAngleRecognizer, _Recognizer);

    function DistanceAngleRecognizer() {
      return _Recognizer.apply(this, arguments) || this;
    }

    var _proto = DistanceAngleRecognizer.prototype;

    _proto.getInternalMovement = function getInternalMovement(values, state) {
      var prev_a = state.values[1]; // not be defined if ctrl+wheel is used for zoom only

      var d = values[0],
          _values$ = values[1],
          a = _values$ === void 0 ? prev_a : _values$;
      var delta_a = a - prev_a;
      var next_turns = state.turns;
      if (Math.abs(delta_a) > 270) next_turns += sign(delta_a);
      return subV([d, a - 360 * next_turns], state.initial);
    };

    _proto.getKinematics = function getKinematics(values, event) {
      var state = this.getMovement(values);
      var turns = (values[1] - state.movement[1] - this.state.initial[1]) / 360;
      var dt = event.timeStamp - this.state.timeStamp;
      var kinematics = calculateAllKinematics(state.movement, state.delta, dt);
      return _extends({
        turns: turns
      }, state, kinematics);
    };

    _proto.mapStateValues = function mapStateValues(state) {
      return {
        da: state.values,
        vdva: state.velocities
      };
    };

    return DistanceAngleRecognizer;
  }(Recognizer);

  var PinchRecognizer = /*#__PURE__*/function (_DistanceAngleRecogni) {
    _inheritsLoose(PinchRecognizer, _DistanceAngleRecogni);

    function PinchRecognizer() {
      var _this;

      _this = _DistanceAngleRecogni.apply(this, arguments) || this;
      _this.ingKey = 'pinching';
      _this.stateKey = 'pinch';

      _this.pinchShouldStart = function (event) {
        var _getGenericEventData = getGenericEventData(event),
            touches = _getGenericEventData.touches;

        return _this.enabled && touches === 2;
      };

      _this.onPinchStart = function (event) {
        if (!_this.pinchShouldStart(event)) return;

        var _getTwoTouchesEventDa = getTwoTouchesEventData(event),
            values = _getTwoTouchesEventDa.values,
            origin = _getTwoTouchesEventDa.origin;

        _this.updateSharedState(getGenericEventData(event));

        _this.updateGestureState(_extends({}, getStartGestureState(_assertThisInitialized(_this), values, event), getGenericPayload(_assertThisInitialized(_this), event, true), {
          cancel: _this.onCancel,
          origin: origin
        }));

        _this.updateGestureState(_this.getMovement(values));

        _this.fireGestureHandler();
      };

      _this.onPinchChange = function (event) {
        var _this$state = _this.state,
            canceled = _this$state.canceled,
            _active = _this$state._active;
        if (canceled || !_active) return;
        var genericEventData = getGenericEventData(event);

        _this.updateSharedState(genericEventData);

        var _getTwoTouchesEventDa2 = getTwoTouchesEventData(event),
            values = _getTwoTouchesEventDa2.values,
            origin = _getTwoTouchesEventDa2.origin; // @ts-ignore


        var kinematics = _this.getKinematics(values, event);

        _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), kinematics, {
          origin: origin
        }));

        _this.fireGestureHandler();
      };

      _this.onPinchEnd = function (event) {
        if (!_this.state.active) return;
        _this.state._active = false;

        _this.updateSharedState({
          down: false,
          touches: 0
        });

        _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getMovement(_this.state.values)));

        _this.fireGestureHandler();
      };

      _this.onCancel = function () {
        if (_this.state.canceled) return;
        _this.state._active = false;

        _this.updateGestureState({
          canceled: true
        });

        _this.updateSharedState({
          down: false,
          touches: 0
        });

        requestAnimationFrame(function () {
          return _this.fireGestureHandler();
        });
      };
      /**
       * PINCH WITH WEBKIT GESTURES
       */


      _this.onGestureStart = function (event) {
        if (!_this.enabled) return;
        event.preventDefault(); // useless

        var values = getWebkitGestureEventValues(event);

        _this.updateSharedState(getGenericEventData(event));

        _this.updateGestureState(_extends({}, getStartGestureState(_assertThisInitialized(_this), values, event), getGenericPayload(_assertThisInitialized(_this), event, true), {
          cancel: _this.onCancel
        }));

        _this.updateGestureState(_this.getMovement(values));

        _this.fireGestureHandler();
      };

      _this.onGestureChange = function (event) {
        var _this$state2 = _this.state,
            canceled = _this$state2.canceled,
            _active = _this$state2._active;
        if (canceled || !_active) return;
        event.preventDefault();
        var genericEventData = getGenericEventData(event);

        _this.updateSharedState(genericEventData);

        var values = getWebkitGestureEventValues(event);

        var kinematics = _this.getKinematics(values, event);

        _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), kinematics));

        _this.fireGestureHandler();
      };

      _this.onGestureEnd = function (event) {
        event.preventDefault();
        if (!_this.state.active) return;
        _this.state._active = false;

        _this.updateSharedState({
          down: false,
          touches: 0
        });

        _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getMovement(_this.state.values)));

        _this.fireGestureHandler();
      };
      /**
       * PINCH WITH WHEEL
       */


      _this.wheelShouldRun = function (event) {
        return _this.enabled && event.ctrlKey;
      };

      _this.getWheelValuesFromEvent = function (event) {
        var _getWheelEventValues = getWheelEventValues(event),
            delta_d = _getWheelEventValues[1];

        var _this$state$values = _this.state.values,
            prev_d = _this$state$values[0],
            prev_a = _this$state$values[1];
        var d = prev_d - delta_d;
        var a = prev_a !== void 0 ? prev_a : 0;
        return {
          values: [d, a],
          origin: [event.clientX, event.clientY],
          delta: [0, delta_d]
        };
      };

      _this.onWheel = function (event) {
        if (!_this.wheelShouldRun(event)) return;

        _this.setTimeout(_this.onWheelEnd);

        if (!_this.state._active) _this.onWheelStart(event);else _this.onWheelChange(event);
      };

      _this.onWheelStart = function (event) {
        var _this$getWheelValuesF = _this.getWheelValuesFromEvent(event),
            values = _this$getWheelValuesF.values,
            delta = _this$getWheelValuesF.delta,
            origin = _this$getWheelValuesF.origin;

        if (event.cancelable) event.preventDefault();else {
          console.warn('To properly support zoom on trackpads, try using the `domTarget` option and `config.eventOptions.passive` set to `false`. This message will only appear in development mode.');
        }

        _this.updateSharedState(getGenericEventData(event));

        _this.updateGestureState(_extends({}, getStartGestureState(_assertThisInitialized(_this), values, event), getGenericPayload(_assertThisInitialized(_this), event, true), {
          initial: _this.state.values,
          offset: values,
          delta: delta,
          origin: origin
        }));

        _this.updateGestureState(_this.getMovement(values));

        _this.fireGestureHandler();
      };

      _this.onWheelChange = function (event) {
        _this.updateSharedState(getGenericEventData(event));

        var _this$getWheelValuesF2 = _this.getWheelValuesFromEvent(event),
            values = _this$getWheelValuesF2.values,
            origin = _this$getWheelValuesF2.origin,
            delta = _this$getWheelValuesF2.delta;

        _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getKinematics(values, event), {
          origin: origin,
          delta: delta
        }));

        _this.fireGestureHandler();
      };

      _this.onWheelEnd = function () {
        _this.state._active = false;

        _this.updateGestureState(_this.getMovement(_this.state.values));

        _this.fireGestureHandler();
      };

      return _this;
    }

    var _proto = PinchRecognizer.prototype;

    _proto.addBindings = function addBindings$1(bindings) {
      // Only try to use gesture events when they are supported and domTarget is set
      // as React doesn't support gesture handlers.
      if (this.controller.config.domTarget && !supportsTouchEvents() && supportsGestureEvents()) {
        addBindings(bindings, 'gestureStart', this.onGestureStart);

        addBindings(bindings, 'gestureChange', this.onGestureChange);

        addBindings(bindings, 'gestureEnd', this.onGestureEnd);
      } else {
        addBindings(bindings, 'touchstart', this.onPinchStart);

        addBindings(bindings, 'touchmove', this.onPinchChange);

        addBindings(bindings, 'touchend', this.onPinchEnd);

        addBindings(bindings, 'touchcancel', this.onPinchEnd);

        addBindings(bindings, 'wheel', this.onWheel);
      }
    };

    return PinchRecognizer;
  }(DistanceAngleRecognizer);

  /**
   * Pinch hook.
   *
   * @param handler - the function fired every time the pinch gesture updates
   * @param [config={}] - the config object including generic options and pinch options
   */

  function usePinch(handler, config) {
    if (config === void 0) {
      config = {};
    }

    RecognizersMap.set('pinch', PinchRecognizer);
    var buildPinchConfig = vueDemi.ref();

    if (!buildPinchConfig.value) {
      buildPinchConfig.value = memoizeOne(_buildPinchConfig, isEqual);
    }

    return useRecognizers({
      pinch: handler
    }, buildPinchConfig.value(config));
  }

  var WheelRecognizer = /*#__PURE__*/function (_CoordinatesRecognize) {
    _inheritsLoose(WheelRecognizer, _CoordinatesRecognize);

    function WheelRecognizer() {
      var _this;

      _this = _CoordinatesRecognize.apply(this, arguments) || this;
      _this.ingKey = 'wheeling';
      _this.stateKey = 'wheel';
      _this.debounced = true;

      _this.handleEvent = function (event) {
        if (event.ctrlKey && 'pinch' in _this.controller.handlers) return;
        if (!_this.enabled) return;

        _this.setTimeout(_this.onEnd);

        _this.updateSharedState(getGenericEventData(event));

        var values = addV(getWheelEventValues(event), _this.state.values);

        if (!_this.state._active) {
          _this.updateGestureState(_extends({}, getStartGestureState(_assertThisInitialized(_this), values, event), getGenericPayload(_assertThisInitialized(_this), event, true), {
            initial: _this.state.values
          }));

          var movement = _this.getMovement(values);

          var geometry = calculateAllGeometry(movement.delta);

          _this.updateGestureState(movement);

          _this.updateGestureState(geometry);
        } else {
          _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getKinematics(values, event)));
        }

        _this.fireGestureHandler();
      };

      _this.onEnd = function () {
        var movement = _this.getMovement(_this.state.values);

        _this.updateGestureState(movement);

        _this.updateGestureState({
          _active: false,
          velocities: [0, 0],
          velocity: 0
        });

        _this.fireGestureHandler();
      };

      return _this;
    }

    var _proto = WheelRecognizer.prototype;

    _proto.addBindings = function addBindings$1(bindings) {
      addBindings(bindings, 'onWheel', this.handleEvent);
    };

    return WheelRecognizer;
  }(CoordinatesRecognizer);

  /**
   * Wheel hook.
   *
   * @param handler - the function fired every time the wheel gesture updates
   * @param the config object including generic options and wheel options
   */

  function useWheel(handler, config) {
    if (config === void 0) {
      config = {};
    }

    RecognizersMap.set('wheel', WheelRecognizer);
    var buildWheelConfig = vueDemi.ref();

    if (!buildWheelConfig.value) {
      buildWheelConfig.value = memoizeOne(_buildWheelConfig, isEqual);
    }

    return useRecognizers({
      wheel: handler
    }, buildWheelConfig.value(config));
  }

  var MoveRecognizer = /*#__PURE__*/function (_CoordinatesRecognize) {
    _inheritsLoose(MoveRecognizer, _CoordinatesRecognize);

    function MoveRecognizer() {
      var _this;

      _this = _CoordinatesRecognize.apply(this, arguments) || this;
      _this.ingKey = 'moving';
      _this.stateKey = 'move';
      _this.debounced = true;

      _this.onMove = function (event) {
        if (!_this.enabled) return;

        _this.setTimeout(_this.onMoveEnd);

        if (!_this.state._active) _this.onMoveStart(event);else _this.onMoveChange(event);
      };

      _this.onMoveStart = function (event) {
        _this.updateSharedState(getGenericEventData(event));

        var values = getPointerEventValues(event);

        _this.updateGestureState(_extends({}, getStartGestureState(_assertThisInitialized(_this), values, event), getGenericPayload(_assertThisInitialized(_this), event, true)));

        _this.updateGestureState(_this.getMovement(values));

        _this.fireGestureHandler();
      };

      _this.onMoveChange = function (event) {
        _this.updateSharedState(getGenericEventData(event));

        var values = getPointerEventValues(event);

        _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getKinematics(values, event)));

        _this.fireGestureHandler();
      };

      _this.onMoveEnd = function () {
        var values = _this.state.values;

        _this.updateGestureState(_this.getMovement(values));

        _this.updateGestureState({
          velocities: [0, 0],
          velocity: 0,
          _active: false
        });

        _this.fireGestureHandler();
      };

      _this.onPointerEnter = function (event) {
        _this.controller.state.shared.hovering = true;
        if (!_this.controller.config.enabled) return;

        if (_this.controller.config.hover.enabled) {
          var values = getPointerEventValues(event);

          var state = _extends({}, _this.controller.state.shared, _this.state, getGenericPayload(_assertThisInitialized(_this), event, true), {
            values: values,
            active: true,
            hovering: true
          });

          _this.controller.handlers.hover(_extends({}, state, _this.mapStateValues(state)));
        }

        if ('move' in _this.controller.handlers) _this.onMoveStart(event);
      };

      _this.onPointerLeave = function (event) {
        _this.controller.state.shared.hovering = false;
        if ('move' in _this.controller.handlers) _this.onMoveEnd();
        if (!_this.controller.config.hover.enabled) return;
        var values = getPointerEventValues(event);

        var state = _extends({}, _this.controller.state.shared, _this.state, getGenericPayload(_assertThisInitialized(_this), event), {
          values: values,
          active: false
        });

        _this.controller.handlers.hover(_extends({}, state, _this.mapStateValues(state)));
      };

      return _this;
    }

    var _proto = MoveRecognizer.prototype;

    _proto.addBindings = function addBindings$1(bindings) {
      if ('move' in this.controller.handlers) {
        addBindings(bindings, 'onPointerMove', this.onMove);
      }

      if ('hover' in this.controller.handlers) {
        addBindings(bindings, 'onPointerEnter', this.onPointerEnter);

        addBindings(bindings, 'onPointerLeave', this.onPointerLeave);
      }
    };

    return MoveRecognizer;
  }(CoordinatesRecognizer);

  /**
   * Move hook.
   *
   * @param handler - the function fired every time the move gesture updates
   * @param [config={}] - the config object including generic options and move options
   */

  function useMove(handler, config) {
    if (config === void 0) {
      config = {};
    }

    RecognizersMap.set('move', MoveRecognizer);
    var buildMoveConfig = vueDemi.ref();

    if (!buildMoveConfig.value) {
      buildMoveConfig.value = memoizeOne(_buildMoveConfig, isEqual);
    }

    return useRecognizers({
      move: handler
    }, buildMoveConfig.value(config));
  }

  /**
   * Hover hook.
   *
   * @param handler - the function fired every time the hover gesture updates
   * @param [config={}] - the config object including generic options and hover options
   */

  function useHover(handler, config) {
    if (config === void 0) {
      config = {};
    }

    RecognizersMap.set('hover', MoveRecognizer);
    var buildHoverConfig = vueDemi.ref();

    if (!buildHoverConfig.value) {
      buildHoverConfig.value = memoizeOne(_buildHoverConfig, isEqual);
    }

    return useRecognizers({
      hover: handler
    }, buildHoverConfig.value(config));
  }

  var ScrollRecognizer = /*#__PURE__*/function (_CoordinatesRecognize) {
    _inheritsLoose(ScrollRecognizer, _CoordinatesRecognize);

    function ScrollRecognizer() {
      var _this;

      _this = _CoordinatesRecognize.apply(this, arguments) || this;
      _this.ingKey = 'scrolling';
      _this.stateKey = 'scroll';
      _this.debounced = true;

      _this.handleEvent = function (event) {
        if (!_this.enabled) return;

        _this.clearTimeout();

        _this.setTimeout(_this.onEnd);

        var values = getScrollEventValues(event);

        _this.updateSharedState(getGenericEventData(event));

        if (!_this.state._active) {
          _this.updateGestureState(_extends({}, getStartGestureState(_assertThisInitialized(_this), values, event), getGenericPayload(_assertThisInitialized(_this), event, true), {
            initial: _this.state.values
          }));

          var movementDetection = _this.getMovement(values);

          var geometry = calculateAllGeometry(movementDetection.delta);

          _this.updateGestureState(movementDetection);

          _this.updateGestureState(geometry);
        } else {
          _this.updateGestureState(_extends({}, getGenericPayload(_assertThisInitialized(_this), event), _this.getKinematics(values, event)));
        }

        _this.fireGestureHandler();
      };

      _this.onEnd = function () {
        _this.state._active = false;

        _this.updateGestureState(_extends({}, _this.getMovement(_this.state.values), {
          velocities: [0, 0],
          velocity: 0
        }));

        _this.fireGestureHandler();
      };

      return _this;
    }

    var _proto = ScrollRecognizer.prototype;

    _proto.addBindings = function addBindings$1(bindings) {
      addBindings(bindings, 'onScroll', this.handleEvent);
    };

    return ScrollRecognizer;
  }(CoordinatesRecognizer);

  /**
   * Scroll hook.
   *
   * @param handler - the function fired every time the scroll gesture updates
   * @param [config={}] - the config object including generic options and scroll options
   */

  function useScroll(handler, config) {
    if (config === void 0) {
      config = {};
    }

    RecognizersMap.set('scroll', ScrollRecognizer);
    var buildScrollConfig = vueDemi.ref();

    if (!buildScrollConfig.value) {
      buildScrollConfig.value = memoizeOne(_buildScrollConfig, isEqual);
    }

    return useRecognizers({
      scroll: handler
    }, buildScrollConfig.value(config));
  }

  var RE_NOT_NATIVE = /^on(Drag|Wheel|Scroll|Move|Pinch|Hover)/;

  function sortHandlers(handlers) {
    var _native = {};
    var handle = {};
    var actions = new Set();

    for (var key in handlers) {
      if (RE_NOT_NATIVE.test(key)) {
        actions.add(RegExp.lastMatch);
        handle[key] = handlers[key];
      } else {
        _native[key] = handlers[key];
      }
    }

    return [handle, _native, actions];
  }
  /**
   * @public
   *
   * The most complete gesture hook, allowing support for multiple gestures.
   *
   * @param {Handlers} handlers - an object with on[Gesture] keys containg gesture handlers
   * @param {UseGestureConfig} [config={}] - the full config object
   * @returns {(...args: any[]) => HookReturnType<Config>}
   */


  function useGesture(_handlers, config) {
    if (config === void 0) {
      config = {};
    }

    var _sortHandlers = sortHandlers(_handlers),
        handlers = _sortHandlers[0],
        nativeHandlers = _sortHandlers[1],
        actions = _sortHandlers[2];

    RecognizersMap.set('drag', DragRecognizer);
    RecognizersMap.set('hover', MoveRecognizer);
    RecognizersMap.set('move', MoveRecognizer);
    RecognizersMap.set('pinch', PinchRecognizer);
    RecognizersMap.set('scroll', ScrollRecognizer);
    RecognizersMap.set('wheel', WheelRecognizer);
    var mergedConfig = buildComplexConfig(config, actions);
    var internalHandlers = {};
    if (actions.has('onDrag')) internalHandlers.drag = includeStartEndHandlers(handlers, 'onDrag');
    if (actions.has('onWheel')) internalHandlers.wheel = includeStartEndHandlers(handlers, 'onWheel');
    if (actions.has('onScroll')) internalHandlers.scroll = includeStartEndHandlers(handlers, 'onScroll');
    if (actions.has('onMove')) internalHandlers.move = includeStartEndHandlers(handlers, 'onMove');
    if (actions.has('onPinch')) internalHandlers.pinch = includeStartEndHandlers(handlers, 'onPinch');
    if (actions.has('onHover')) internalHandlers.hover = handlers.onHover;
    return useRecognizers(internalHandlers, mergedConfig, nativeHandlers);
  }

  function includeStartEndHandlers(handlers, handlerKey) {
    var startKey = handlerKey + 'Start';
    var endKey = handlerKey + 'End';

    var fn = function fn(state) {
      var memo = undefined;
      if (state.first && startKey in handlers) handlers[startKey](state);
      if (handlerKey in handlers) memo = handlers[handlerKey](state);
      if (state.last && endKey in handlers) handlers[endKey](state);
      return memo;
    };

    return fn;
  }

  var noWobble = {
    mass: 1,
    tension: 170,
    friction: 26,
    precision: 0.01
  }; // the default, if nothing provided

  var gentle = {
    mass: 1,
    tension: 120,
    friction: 14,
    precision: 0.01
  };
  var wobbly = {
    mass: 1,
    tension: 180,
    friction: 12,
    precision: 0.01
  };
  var stiff = {
    mass: 1,
    tension: 210,
    friction: 20,
    precision: 0.01
  };
  var presets = {
    noWobble: noWobble,
    gentle: gentle,
    wobbly: wobbly,
    stiff: stiff
  };

  var isClient = typeof window !== 'undefined';
  var raf = isClient ? /*#__PURE__*/window.requestAnimationFrame.bind(window) : function () {};
  var cancelRaf = isClient ? /*#__PURE__*/window.cancelAnimationFrame.bind(window) : function () {};
  var now = typeof performance !== 'undefined' ? /*#__PURE__*/performance.now.bind(performance) : /*#__PURE__*/Date.now.bind(Date);
  var isArray = /*#__PURE__*/Array.isArray.bind(Array);
  var isFunction = function isFunction(value) {
    return typeof value === 'function';
  }; // stepper is used a lot. Saves allocation to return the same array wrapper.
  // This is fine and danger-free against mutations because the callsite
  // immediately destructures it and gets the numbers inside without passing the
  // array reference around.

  var reusedTuple = [0, 0];
  function stepper(secondPerFrame, x, v, destX, spring) {
    // Spring stiffness, in kg / s^2
    // for animations, destX is really spring length (spring at rest). initial
    // position is considered as the stretched/compressed position of a spring
    var Fspring = -spring.tension * (x - destX); // Damping, in kg / s

    var Fdamper = -spring.friction * v;
    var a = (Fspring + Fdamper) / (spring.mass || 1);
    var newV = v + a * secondPerFrame;
    var newX = x + newV * secondPerFrame;
    var precision = spring.precision || 0.01;

    if (Math.abs(newV) < precision && Math.abs(newX - destX) < precision) {
      reusedTuple[0] = destX;
      reusedTuple[1] = 0;
      return reusedTuple;
    }

    reusedTuple[0] = newX;
    reusedTuple[1] = newV;
    return reusedTuple;
  }

  var msPerFrame = 1000 / 60;

  function defineInitialValues(values, velocities) {
    var newValues = isArray(values) ? [] : {};
    var newVelocities = isArray(values) ? [] : {};

    for (var key in values) {
      newValues[key] = values[key]; // @ts-ignore

      newVelocities[key] = velocities ? velocities[key] : 0;
    }

    return [newValues, newVelocities];
  }

  var noop$1 = function noop() {};

  function useSpring(initialValue, // TODO: could change
  springConfiguration, options) {
    if (options === void 0) {
      options = {};
    }

    var spring = springConfiguration || noWobble;
    var onRest = options.onRest || noop$1; // hold a ref to real values so we can change them
    // @ts-ignore

    var realValues = isFunction(initialValue) ? vueDemi.ref(initialValue()) : vueDemi.ref(initialValue);
    var wasAnimating = false;
    var prevTime = 0;
    var accumulatedTime = 0;
    vueDemi.watch(realValues, function (_current, _old) {
      if (!wasAnimating) {
        prevTime = now();
        accumulatedTime = 0;
        animate();
      }
    }, {
      deep: true
    });
    var current = defineInitialValues(realValues.value, null);
    var currentValues = vueDemi.ref(current[0]);
    var currentVelocities = vueDemi.ref(current[1]); // only done to correctly infer types, they're overridden in mounted

    var idealValues = current[0];
    var idealVelocities = current[1];
    vueDemi.onMounted(function () {
      prevTime = now();
      accumulatedTime = 0;
      var ideal = defineInitialValues(currentValues.value, currentVelocities.value);
      idealValues = ideal[0];
      idealVelocities = ideal[1];
      animate();
    });
    var animationId; // TODO: also cancel when a new value comes

    vueDemi.onUnmounted(function () {
      if (animationId) {
        cancelRaf(animationId);
      }
    });

    function animate() {
      animationId = raf(function () {
        if (shouldStopAnimation(currentValues.value, realValues.value, currentVelocities.value)) {
          // TODO: emit
          if (wasAnimating) onRest(); // reset everything for next animation

          animationId = null;
          wasAnimating = false;
          return;
        } // TODO: emit
        // if (!wasAnimating) this.$emit('motion-start')


        wasAnimating = true; // get time from last frame

        var currentTime = now();
        var timeDelta = currentTime - prevTime;
        prevTime = currentTime;
        accumulatedTime += timeDelta; // more than 10 frames? probably switched browser tab. Restart

        if (accumulatedTime > msPerFrame * 10) {
          accumulatedTime = 0;
        }

        if (accumulatedTime === 0) {
          // no need to cancel animationID here; shouldn't have any in flight
          animationId = null; // TODO: emit
          // this.$emit('motion-restart')

          return animate();
        }

        var currentFrameCompletion = (accumulatedTime - Math.floor(accumulatedTime / msPerFrame) * msPerFrame) / msPerFrame;
        var framesToCatchUp = Math.floor(accumulatedTime / msPerFrame);
        animateValues(framesToCatchUp, currentFrameCompletion, spring, realValues.value, currentValues.value, currentVelocities.value, idealValues, idealVelocities); // out of the update loop

        animationId = null; // the amount we're looped over above

        accumulatedTime -= framesToCatchUp * msPerFrame; // keep going!

        animate();
      });
    }

    var modifiableValues = isArray(realValues.value) ? [] : {};

    var _loop = function _loop(key) {
      // @ts-ignore
      modifiableValues[key] = vueDemi.computed({
        // @ts-ignore
        get: function get() {
          return currentValues.value[key];
        },
        // @ts-ignore
        set: function set(v) {
          return realValues.value[key] = v;
        }
      });
    };

    for (var key in realValues.value) {
      _loop(key);
    }

    if (isFunction(initialValue)) {
      // @ts-ignore
      var values = vueDemi.toRefs(vueDemi.reactive(modifiableValues));

      var update = function update(rawValuesObject) {
        var keys = Object.keys(rawValuesObject);
        keys.forEach(function (key) {
          if (!(key in values)) return;
          values[key].value = rawValuesObject[key];
          if ('config' in rawValuesObject) Object.assign(spring, {
            config: rawValuesObject.config
          });
          if ('immediate' in rawValuesObject) Object.assign(spring, {
            immediate: rawValuesObject.immediate
          });
        });
      }; // @ts-ignore


      return [values, update];
    }

    return vueDemi.reactive(modifiableValues);
  }

  function shouldStopAnimation(currentValues, values, currentVelocities) {
    for (var key in values) {
      // istanbul ignore if
      // if (!Object.prototype.hasOwnProperty.call(values, key)) continue
      // Something is still moving
      if (currentVelocities[key] !== 0) return false; // Something hasn't reached its destination
      // stepper will have already taken care of rounding precision errors, so
      // won't have such thing as 0.9999 != 1

      if (currentValues[key] !== values[key]) return false;
    }

    return true;
  }

  function animateValues(framesToCatchUp, currentFrameCompletion, spring, realValues, currentValues, currentVelocities, idealValues, idealVelocities) {
    for (var key in realValues) {
      // istanbul ignore if
      // if (!Object.prototype.hasOwnProperty.call(realValues, key)) continue
      var newIdealValue = idealValues[key];
      var newIdealVelocity = idealVelocities[key];
      var value = realValues[key]; // iterate as if the animation took place

      for (var i = 0; i < framesToCatchUp; i++) {

        var _stepper = stepper(msPerFrame / 1000, newIdealValue, newIdealVelocity, value, spring);

        newIdealValue = _stepper[0];
        newIdealVelocity = _stepper[1];
      }

      var _stepper2 = stepper(msPerFrame / 1000, newIdealValue, newIdealVelocity, value, spring),
          nextIdealValue = _stepper2[0],
          nextIdealVelocity = _stepper2[1]; // @ts-ignore


      currentValues[key] = newIdealValue + (nextIdealValue - newIdealValue) * currentFrameCompletion; // @ts-ignore

      currentVelocities[key] = newIdealVelocity + (nextIdealVelocity - newIdealVelocity) * currentFrameCompletion; // @ts-ignore

      idealValues[key] = newIdealValue; // @ts-ignore

      idealVelocities[key] = newIdealVelocity;
    }
  }

  exports.addV = addV;
  exports.gentle = gentle;
  exports.noWobble = noWobble;
  exports.presets = presets;
  exports.rubberbandIfOutOfBounds = rubberbandIfOutOfBounds;
  exports.stiff = stiff;
  exports.subV = subV;
  exports.useDrag = useDrag;
  exports.useGesture = useGesture;
  exports.useHover = useHover;
  exports.useMove = useMove;
  exports.usePinch = usePinch;
  exports.useScroll = useScroll;
  exports.useSpring = useSpring;
  exports.useWheel = useWheel;
  exports.wobbly = wobbly;

  Object.defineProperty(exports, '__esModule', { value: true });

  let GlobalVue = null
  if (typeof window !== 'undefined') {
    GlobalVue = window.Vue
  } else if (typeof global !== 'undefined') {
    GlobalVue = global.Vue
  }
  if (GlobalVue) {
    GlobalVue.component("VueUseGesture", VueUseGesture)
  }

})));
//# sourceMappingURL=vueusegesture.umd.development.js.map
